///<reference path="../../.d.ts"/>
"use strict";
var broccoli = require('broccoli');
var path = require('path');
var Future = require("fibers/future");
var tree_differ_1 = require('./tree-differ');
var Builder = (function () {
    function Builder($fs, $nodeModulesTree, $projectDataService, $logger) {
        this.$fs = $fs;
        this.$nodeModulesTree = $nodeModulesTree;
        this.$projectDataService = $projectDataService;
        this.$logger = $logger;
    }
    Builder.prototype.prepareNodeModules = function (absoluteOutputPath, projectDir) {
        var _this = this;
        return (function () {
            _this.$projectDataService.initialize(projectDir);
            var cachedNodeModulesPath = _this.$projectDataService.getValue("node-modules-cache-path").wait();
            if (cachedNodeModulesPath && _this.$fs.exists(cachedNodeModulesPath).wait()) {
                var diffTree = new tree_differ_1.TreeDiffer(cachedNodeModulesPath);
                var diffTreeResult = diffTree.diffTree(path.join(projectDir, absoluteOutputPath, "node_modules"));
                if (diffTreeResult.changedDirectories.length > 0 || diffTreeResult.removedDirectories.length > 0) {
                    _this.rebuildNodeModulesTree(absoluteOutputPath, projectDir).wait();
                }
            }
            else {
                _this.rebuildNodeModulesTree(absoluteOutputPath, projectDir).wait();
            }
        }).future()();
    };
    Builder.prototype.rebuildNodeModulesTree = function (outputPath, projectDir) {
        var nodeModulesBuilder = this.makeNodeModulesBuilder(outputPath, projectDir);
        return this.rebuild(nodeModulesBuilder);
    };
    Builder.prototype.makeNodeModulesBuilder = function (outputPath, projectDir) {
        var tree = this.$nodeModulesTree.makeNodeModulesTree(outputPath, projectDir);
        return new broccoli.Builder(tree);
    };
    Builder.prototype.rebuild = function (builder) {
        var _this = this;
        var future = new Future();
        builder.build()
            .then(function (result) {
            future.return(result);
        })
            .catch(function (err) {
            if (err.file) {
                _this.$logger.error("File: " + err.file);
            }
            if (err.stack) {
                _this.$logger.error(err.stack);
            }
            future.throw(err);
        });
        return future;
    };
    return Builder;
})();
exports.Builder = Builder;
$injector.register("broccoliBuilder", Builder);
