///<reference path="../../.d.ts"/>
"use strict";
var fs = require("fs");
var path = require('path');
var semver = require("semver");
var shelljs = require("shelljs");
var broccoli_plugin_wrapper_factory_1 = require('./broccoli-plugin-wrapper-factory');
var constants = require("./../../constants");
var DestCopy = (function () {
    function DestCopy(inputPath, cachePath, outputRoot, projectDir) {
        this.inputPath = inputPath;
        this.cachePath = cachePath;
        this.outputRoot = outputRoot;
        this.projectDir = projectDir;
    }
    DestCopy.prototype.rebuild = function (treeDiff) {
        var _this = this;
        var dependencies = this.getDependencies();
        var devDependencies = this.getDevDependencies(this.projectDir);
        treeDiff.changedDirectories.forEach(function (changedDirectory) {
            var changedDirectoryAbsolutePath = path.join(_this.inputPath, constants.NODE_MODULES_FOLDER_NAME, changedDirectory);
            var packageJsonFiles = [path.join(changedDirectoryAbsolutePath, "package.json")];
            var nodeModulesFolderPath = path.join(changedDirectoryAbsolutePath, "node_modules");
            packageJsonFiles = packageJsonFiles.concat(_this.enumeratePackageJsonFilesSync(nodeModulesFolderPath));
            _.each(packageJsonFiles, function (packageJsonFilePath) {
                var fileContent = require(packageJsonFilePath);
                var isPlugin = fileContent.nativescript;
                if (!devDependencies[fileContent.name]) {
                    var currentDependency = {
                        name: fileContent.name,
                        version: fileContent.version,
                        directory: path.dirname(packageJsonFilePath),
                        isPlugin: isPlugin
                    };
                    var addedDependency = dependencies[currentDependency.name];
                    if (addedDependency) {
                        if (semver.gt(currentDependency.version, addedDependency.version)) {
                            var currentDependencyMajorVersion = semver.major(currentDependency.version);
                            var addedDependencyMajorVersion = semver.major(addedDependency.version);
                            var message = "The depedency located at " + addedDependency.directory + " with version  " + addedDependency.version + " will be replaced with dependency located at " + currentDependency.directory + " with version " + currentDependency.version;
                            var logger = $injector.resolve("$logger");
                            currentDependencyMajorVersion === addedDependencyMajorVersion ? logger.out(message) : logger.warn(message);
                            dependencies[currentDependency.name] = currentDependency;
                        }
                    }
                    else {
                        dependencies[currentDependency.name] = currentDependency;
                    }
                }
            });
        });
        _.each(dependencies, function (dependency) {
            shelljs.cp("-R", dependency.directory, _this.outputRoot);
            shelljs.rm("-rf", path.join(_this.outputRoot, dependency.name, "node_modules"));
            if (dependency.isPlugin) {
                shelljs.rm("-rf", path.join(_this.outputRoot, dependency.name, "platforms"));
            }
        });
        var projectFilePath = path.join(this.projectDir, constants.PACKAGE_JSON_FILE_NAME);
        var projectFileContent = require(projectFilePath);
        projectFileContent[constants.NATIVESCRIPT_KEY_NAME][constants.NODE_MODULE_CACHE_PATH_KEY_NAME] = this.inputPath;
        fs.writeFileSync(projectFilePath, JSON.stringify(projectFileContent, null, "\t"), { encoding: "utf8" });
    };
    DestCopy.prototype.getDependencies = function () {
        var result = Object.create(null);
        if (fs.existsSync(this.outputRoot)) {
            var dirs = fs.readdirSync(this.outputRoot);
            _.each(dirs, function (dir) {
                var filePath = path.join(dir, constants.PACKAGE_JSON_FILE_NAME);
                if (fs.existsSync(filePath)) {
                    var fileContent = require(filePath);
                    result[fileContent.name] = fileContent;
                }
            });
        }
        return result;
    };
    DestCopy.prototype.getDevDependencies = function (projectDir) {
        var projectFilePath = path.join(projectDir, constants.PACKAGE_JSON_FILE_NAME);
        var projectFileContent = require(projectFilePath);
        return projectFileContent.devDependencies || {};
    };
    DestCopy.prototype.enumeratePackageJsonFilesSync = function (nodeModulesDirectoryPath, foundFiles) {
        foundFiles = foundFiles || [];
        if (fs.existsSync(nodeModulesDirectoryPath)) {
            var contents = fs.readdirSync(nodeModulesDirectoryPath);
            for (var i = 0; i < contents.length; ++i) {
                foundFiles.push(path.join(nodeModulesDirectoryPath, contents[i], "package.json"));
                var directoryPath = path.join(nodeModulesDirectoryPath, contents[i], "node_modules");
                if (fs.existsSync(directoryPath)) {
                    this.enumeratePackageJsonFilesSync(directoryPath, foundFiles);
                }
            }
        }
        return foundFiles;
    };
    return DestCopy;
})();
exports.DestCopy = DestCopy;
exports.default = broccoli_plugin_wrapper_factory_1.wrapBroccoliPlugin(DestCopy);
